use std::collections::BTreeMap;

use super::{DamageType, PossessionData, PossessionType, SoakData, WearData};
use crate::static_content::species::BodyPart;
use once_cell::sync::OnceCell;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| vec!(
        (
            PossessionType::Shirt,
            PossessionData {
                display: "shirt",
                details: "A fairly plain shirt that probably provides no protection against anything except fashion crimes",
                aliases: vec!("shirt"),
                weight: 70,
                wear_data: Some(WearData {
                    covers_parts: vec!(BodyPart::Arms,
                                       BodyPart::Chest,
                                       BodyPart::Back),
                    thickness: 2.0,
                    dodge_penalty: 0.0,
                    soaks: BTreeMap::new(),
                }),
                ..Default::default()
            }
        ),
        (
            PossessionType::LeatherJacket,
            PossessionData {
                display: "leather jacket",
                details: "A black leather jacket that looks like it would protect you from falling off a motorbike, or maybe even offer some protection against certain weapons",
                aliases: vec!("jacket"),
                weight: 600,
                wear_data: Some(WearData {
                    covers_parts: vec!(BodyPart::Arms,
                                       BodyPart::Chest,
                                       BodyPart::Back),
                    thickness: 4.0,
                    dodge_penalty: 0.3,
                    soaks: vec!(
                        (DamageType::Beat,
                         SoakData {
                             min_soak: 1.0,
                             max_soak: 2.0,
                             damage_probability_per_soak: 0.05
                         }
                        ),
                        (DamageType::Slash,
                         SoakData {
                             min_soak: 2.0,
                             max_soak: 3.0,
                             damage_probability_per_soak: 0.05
                         }
                        ),
                        (DamageType::Pierce,
                         SoakData {
                             min_soak: 2.0,
                             max_soak: 3.0,
                             damage_probability_per_soak: 0.05
                         }
                        ),
                    ).into_iter().collect(),
                }),
                ..Default::default()
            }
        ),
        (
            PossessionType::RadSuit,
            PossessionData {
                display: "radiation suit",
                details: "A suit with a yellow outer layer and a foil lining that covers every part of the body from head to toe, featuring a tight fitting respirator, a hood that covers the head, gloves, plastic overalls, and socks. It seems designed not to let anything in from the outside except through the filter of the respirator, although it probably will hinder movement and won't protect against anything except chemicals or radiation",
                aliases: vec!("rad suit", "radsuit"),
                weight: 2000,
                wear_data: Some(WearData {
                    covers_parts: vec!(
                        BodyPart::Head,
                        BodyPart::Face,
                        BodyPart::Hands,
                        BodyPart::Arms,
                        BodyPart::Chest,
                        BodyPart::Back,
                        BodyPart::Groin,
                        BodyPart::Legs,
                        BodyPart::Feet
                    ),
                    thickness: 4.0,
                    dodge_penalty: 1.0,
                    soaks: vec!(
                        (DamageType::Radiation,
                         SoakData {
                             min_soak: 500.0,
                             max_soak: 600.0,
                             damage_probability_per_soak: 0.01
                         }
                        ),
                    ).into_iter().collect(),
                }),
                ..Default::default()
            }
        ),
        (
            PossessionType::ShieldWeaveJacket,
            PossessionData {
                display: "ShieldWeave jacket",
                details: "A fancy-looking black jacket that could be formal business wear, except it is made out of some kind of light but extremely strong fibre. A label inside says it is a ShieldWeave bullet-resistant business jacket, offering stylish protection against fire from pistols, as well as offering a degree of stab and slash protection",
                aliases: vec!("shield weave jacket", "jacket", "shieldweave jacket"),
                weight: 300,
                wear_data: Some(WearData {
                    covers_parts: vec!(BodyPart::Arms,
                                       BodyPart::Chest,
                                       BodyPart::Back),
                    thickness: 4.0,
                    dodge_penalty: 0.25,
                    soaks: vec!(
                        (DamageType::Bullet,
                         SoakData {
                             min_soak: 60.0,
                             max_soak: 70.0,
                             damage_probability_per_soak: 0.01
                         }
                        ),
                        (DamageType::Slash,
                         SoakData {
                             min_soak: 2.0,
                             max_soak: 3.0,
                             damage_probability_per_soak: 0.01
                         }
                        ),
                        (DamageType::Pierce,
                         SoakData {
                             min_soak: 2.0,
                             max_soak: 3.0,
                             damage_probability_per_soak: 0.01
                         }
                        ),
                    ).into_iter().collect(),
                }),
                ..Default::default()
            }
        ),
    ))
}
